from .utils import Dataset


class FlickrLDL(Dataset):
    def __init__(self, root='external/Flickr_LDL', train=True, transform=None):
        super().__init__(root, train, transform)


class TwitterLDL(Dataset):
    def __init__(self, root='external/Twitter_LDL', train=True, transform=None):
        super().__init__(root, train, transform)



from .utils import Dataset


class IAPS(Dataset):
    label_types = ['vad']

    def __init__(self, root='external/IAPS/IAPS', train=True, transform=None):
        super().__init__(root, train, transform)

    def __loadtarget__(self, root):
        self.ys = {'vad': []}
        for x in self.__readgt__(root, spliter='\t', start_line=0, suffix='.jpg'):
            y = [float(_str.replace('\n', '')) for _str in x[1:]]
            self.ys['vad'].append(y)


class NAPS(IAPS):
    def __init__(self, root='external/IAPS/NAPS', train=True, transform=None):
        super().__init__(root, train, transform)



import os
import numpy as np
import random
from .utils import Dataset
import json


def __maxidx__(_list):
    return _list.index(max(_list))


class FI(Dataset):
    def __init__(self, root='external/FI', train=True, transform=None):
        super().__init__(root, train, transform)

    def __gensplit__(self, root, img_dir='images', seed=0):
        np.random.seed(seed)
        _list = [x for x in os.listdir(f'{root}/{img_dir}')]
        random.shuffle(_list)
        data = {}
        for x in _list:
            label = x.split('_')[0]
            if label not in data.keys():
                data[label] = []
            data[label].append(x)
        _dict = {'train': [], 'test': []}
        for key in data.keys():
            _list = data[key]
            pivod = int(len(_list)*0.15)
            _dict['train'].extend(_list[pivod:])
            _dict['test'].extend(_list[:pivod])
        random.shuffle(_dict['train'])
        random.shuffle(_dict['test'])
        with open(f'{root}/split.json', 'w') as f:
            json.dump(_dict, f)

    def __loadtarget__(self, root, spliter=' ', start_line=1):
        labels = ['amusement', 'awe', 'contentment', 'excitement', 'anger', 'disgust', 'fear', 'sadness']
        for x in self.data:
            label = x.split('/')[-1].split('_')[0]
            y = [1 if _str == label else 0 for _str in labels]
            self.ys['single'].append(__maxidx__(y))
            self.ys['distribution'].append(y)



import matplotlib.pyplot as plt
import shutil
import os
import cv2
import numpy as np
import torch

import sys
sys.path.append('.')


class Visualize():
    def __init__(self, log_path='external/rec.log'):
        """
            simple visualization utils for dirty logs,
            Args:
                log_path: path to .log file.
        """
        self._hash = 'unknown'
        self.figure_type = 'curve'
        self.xs = []
        self.clear()
        self.legends = {}
        self.log_path = log_path

    def __initas__(self, figure_type='curve'):
        if figure_type == 'curve':
            fig, acc_ax = plt.subplots()
            self.acc_ax = acc_ax
            self.loss_ax = acc_ax.twinx()
            self.acc_ax.set_xlabel("epoches")
            self.acc_ax.set_ylabel("accuracy")
            self.loss_ax.set_ylabel("loss")
        elif figure_type == 'image':
            pass

    def clear(self):
        """
            clear plt canvas.
        """
        plt.close('all')

    def import_hash(self, _hash):
        """
            import autocontrol record hash for visualization,
            Args:
                _hash:
        """
        self._hash = _hash.replace('<', '').replace('>', '')
        return self

    def curve(self, _type='acc', smooth=0, legend=None, color=None, alpha=1):
        """
            add acc/loss curve to canvas,
            Args:
        """
        x = self.__loadcurve__(self._hash, _type)
        self.xs.append({'type': _type, 'data': self.__smooth__(x, smooth), 'color': color, 'alpha': alpha})
        self.legend(self._hash if not legend else legend, _type)
        return self

    @staticmethod
    def __smooth__(x, weight):
        scalar = [_y for (_x, _y) in x]
        last = scalar[0]
        smoothed = []
        for point in scalar:
            smoothed_val = last * weight + (1 - weight) * point
            smoothed.append(smoothed_val)
            last = smoothed_val
        smoothed = [(x[i][0], _y) for i, _y in enumerate(smoothed)]
        return smoothed

    def plot(self, display_loss_legend=True):
        """
            build the image from canvas,
            Args:
                display_loss_legend:
        """
        if self.figure_type == 'curve':
            def __read__(data, dim=0):
                return [_x[dim] for _x in data]
            self.__initas__('curve')
            for x in self.xs:
                _type = x['type']
                data = x['data']
                color = x['color']
                alpha = x['alpha']
                if 'loss' in _type:
                    self.loss_ax.plot(__read__(data, 0), __read__(data, 1), color=color, alpha=alpha)
                else:
                    self.acc_ax.plot(__read__(data, 0), __read__(data, 1), color=color, alpha=alpha)
            self.__legend__(display_loss_legend)
            return self
        elif self.figure_type == 'image':
            self.__initas__('image')
            x = self.xs[0]
            self.__imshow__(x, size=x.shape[:2])
            return self

    @staticmethod
    def __imshow__(x, size=(448, 448), annotation=''):
        fig = plt.gcf()  # generate outputs
        plt.imshow(x, aspect='equal'), plt.axis('off'), fig.set_size_inches(size[1]*12/300.0, size[0]*12/300.0)
        plt.gca().xaxis.set_major_locator(plt.NullLocator()), plt.gca().yaxis.set_major_locator(plt.NullLocator()), plt.subplots_adjust(top=1, bottom=0, left=0, right=1, hspace=0, wspace=0), plt.margins(0, 0)
        plt.text(0, 0, annotation, color='white', size=4, ha="left", va="top", bbox=dict(boxstyle="square", ec='black', fc='black'))
        # plt.savefig(path, dpi=300, pad_inches=0)    # visualize masked image

    def __select__(self, _hash):
        with open(self.log_path, 'r') as f:
            for line in f.readlines():
                if 'testing#' not in line and _hash in line:
                    yield line

    def __readepoch__(self, line):
        """
            read float epoch from line.
            Args:
                line: (str).
        """
        epoch, step = line.split('[')[-1].split(']')[0].split(',')
        if '-' in step:
            return int(epoch) + 1
        else:
            step, total_step = step.strip().split('/')
            return int(epoch) + float(step)/float(total_step)

    def __loadcurve__(self, _hash, loss_type='train_loss'):
        x = []
        for line in self.__select__(_hash):
            if f'{loss_type}:' in line:
                data = float(line.split(':')[-1].strip())
                x.append((self.__readepoch__(line), data))
        return self.__cleancurve__(x)

    def __cleancurve__(self, x):
        last_epoch = -1
        for idx, (epoch, data) in enumerate(x):
            if epoch < last_epoch:
                x = self.__rmdupcurve__(x, idx)
                return self.__cleancurve__(x)
            else:
                last_epoch = epoch
        return x

    def __rmdupcurve__(self, x, idx):
        new_x = []
        dup_start = x[idx][0]
        for i, (epoch, data) in enumerate(x):
            if epoch >= dup_start and i < idx:
                continue
            new_x.append((epoch, data))
        return new_x

    def legend(self, labels, ax_type='acc'):
        """
            add legends for acc/loss curve,
            Args:
                labels: str / list[str]
                ax_type:
        """
        if ax_type not in self.legends.keys():
            self.legends[ax_type] = []
        if type(labels) is str:
            labels = [labels]
        self.legends[ax_type].extend(labels)
        return self

    def __legend__(self, display_loss_legend):
        for ax_type in self.legends:
            if ax_type == 'acc':
                self.acc_ax.legend(self.legends[ax_type])
            elif 'loss' in ax_type and display_loss_legend:
                self.loss_ax.legend(self.legends[ax_type])

    def save(self, path=None):
        """
            save image to path.
            Args:
                path:
        """
        if path is None:
            path = f'build/{self._hash}-{self.figure_type}.png'
        plt.show()
        plt.savefig(path, pad_inches=0)
        print(f'saved as {path}')
        return self

    def from_tensor(self, x):
        """
            read tensor as image,
            1-dim tensors are displayed as heatmap, see __heatmap__().
            Args:
                x: input tensor.
        """
        if self.figure_type != 'image':
            self.xs = []
            self.figure_type = 'image'
        if x.shape[0] == 1:
            self.xs.append(self.__heatmap__(x))
        else:
            self.xs.append(self.__tensor2img__(x))
        return self

    @staticmethod
    def __heatmap__(x):
        heatmap = x[0].cpu().numpy()
        heatmap = heatmap/np.max(heatmap)
        # must convert to type unit8
        heatmap = np.uint8(255 * heatmap)
        heatmap = cv2.applyColorMap(heatmap, cv2.COLORMAP_JET)
        return heatmap

    @staticmethod
    def __tensor2img__(x, imtype=np.uint8):
        """"将tensor的数据类型转成numpy类型，并反归一化.

        Parameters:
            input_image (tensor) --  输入的图像tensor数组
            imtype (type)        --  转换后的numpy的数据类型
        """
        mean = [0.485, 0.456, 0.406]
        std = [0.229, 0.224, 0.225]

        if not isinstance(x, np.ndarray):
            if isinstance(x, torch.Tensor):  # get the data from a variable
                image_tensor = x.data
            else:
                return x
            image_numpy = image_tensor.cpu().float().numpy()  # convert it into a numpy array
            if image_numpy.shape[0] == 1:  # grayscale to RGB
                image_numpy = np.tile(image_numpy, (3, 1, 1))
            for i in range(len(mean)):
                image_numpy[i] = image_numpy[i] * std[i] + mean[i]
            image_numpy = image_numpy * 255
            image_numpy = np.transpose(image_numpy, (1, 2, 0))  # post-processing: tranpose and scaling
        else:  # if it is a numpy array, do nothing
            image_numpy = x
        return image_numpy.astype(imtype)

    def __makegif__(self, from_dir='build/cache', fp='build/test.gif'):
        """
            not implemented yet.
        """
        # gif_images = [imageio.imread(f'{from_dir}/{img_file}') for img_file in files]
        # imageio.mimsave(fp, gif_images, fps=8)
        raise NotImplementedError

    def _clean_cache(self, target_dir='build/cache'):
        """
            clean cache dir. (can be dangerous)
            Args:
                target_dir:
        """
        if os.path.exists(target_dir):
            shutil.rmtree(target_dir)
        os.mkdir(target_dir)
        print(f'cache dir {target_dir} cleaned.')



from .utils import Dataset


def __maxidx__(_list):
    return _list.index(max(_list))


class EmotionROI(Dataset):
    label_types = ['single', 'distribution', 'va']

    def __init__(self, root='external/EmotionROI', train=True, transform=None):
        super().__init__(root, train, transform)

    def __loadtarget__(self, root):
        for x in self.__readgt__(root, spliter='\t'):
            y1 = [float(_str.replace('\n', '')) for _str in x[3:]]
            y2 = [float(_str.replace('\n', '')) for _str in x[1:3]]
            self.ys['single'].append(__maxidx__(y1))
            self.ys['distribution'].append(y1)
            self.ys['va'].append(y2)



from .utils import Dataset
from .ldl import FlickrLDL, TwitterLDL
from .iaps import IAPS, NAPS
from .emotion_roi import EmotionROI
from .emod import EMOd
from .fi import FI

__all__ = [
    'Dataset', 'FlickrLDL', 'TwitterLDL', 'IAPS', 'NAPS', 'EmotionROI', 'EMOd', 'FI',
]



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: klrc
# Mail: sh@mail.ecust.edu.cn
#############################################

from setuptools import setup, find_packages  # 这个包没有的可以pip一下

setup(
    name="labvision",  # 这里是pip项目发布的名称
    version="0.0.12",  # 版本号，数值大的会优先被pip
    keywords=("pip", "labvision", "ecust"),
    description="ecust vision lab platform",
    long_description="experimental platform for CV research.",
    license="MIT Licence",

    url="https://github.com/klrc/labvision",  # 项目相关文件地址，一般是github
    author="klrc",
    author_email="sh@mail.ecust.edu.com",

    packages=find_packages(),
    include_package_data=True,
    platforms=["all"],
    install_requires=["torch", "torchvision", "opencv-python", "numpy", "scipy"]  # 这个项目需要的第三方库
)


'''
    >>> python setup.py sdist
    >>> twine upload dist/labvision-0.0.x.tar.gz
    >>> pip install labvision -U -i https://pypi.org/simple 
'''



from torchvision.transforms import transforms


class ChainedTransform():

    def __init__(self):
        """
            chained tranform for torchvision.transforms.
            example_1:
                >>> def basic_normalize(**args):
                >>>    x = ChainedTransform()
                >>>    x = x.ToPILImage().ToTensor().Normalize(**args)
                >>>    return x
            example_2:
                >>> transf = ChainedTransform()
                >>> transf = transf.ToPILImage().ToTensor()
        """
        self.transforms = []
        self._compiled_transform = None

    def __call__(self, pic):
        if self._compiled_transform is None:
            self._compile()
        return self._compiled_transform(pic)

    def _add(self, transform):
        self.transforms.append(transform)
        return self

    def _compile(self):
        t = transforms.Compose(self.transforms)
        self._compiled_transform = t
        return t

    def ToTensor(self, **args):
        return self._add(transforms.ToTensor(**args))

    def ToPILImage(self, **args):
        return self._add(transforms.ToPILImage(**args))

    def Normalize(self, mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225], **args):
        '''
            default mean & std as ImageNet.
        '''
        return self._add(transforms.Normalize(mean=mean, std=std, **args))

    def Resize(self, **args):
        return self._add(transforms.Resize(**args))

    def Scale(self, **args):
        return self._add(transforms.Scale(**args))

    def CenterCrop(self, **args):
        return self._add(transforms.CenterCrop(**args))

    def Pad(self, **args):
        return self._add(transforms.Pad(**args))

    def Lambda(self, **args):
        return self._add(transforms.Lambda(**args))

    def RandomApply(self, **args):
        return self._add(transforms.RandomApply(**args))

    def RandomChoice(self, **args):
        return self._add(transforms.RandomChoice(**args))

    def RandomOrder(self, **args):
        return self._add(transforms.RandomOrder(**args))

    def RandomCrop(self, **args):
        return self._add(transforms.RandomCrop(**args))

    def RandomHorizontalFlip(self, **args):
        return self._add(transforms.RandomHorizontalFlip(**args))

    def RandomVerticalFlip(self, **args):
        return self._add(transforms.RandomVerticalFlip(**args))

    def RandomResizedCrop(self, **args):
        return self._add(transforms.RandomResizedCrop(**args))

    def RandomSizedCrop(self, **args):
        return self._add(transforms.RandomSizedCrop(**args))

    def FiveCrop(self, **args):
        return self._add(transforms.FiveCrop(**args))

    def TenCrop(self, **args):
        return self._add(transforms.TenCrop(**args))

    def LinearTransformation(self, **args):
        return self._add(transforms.LinearTransformation(**args))

    def ColorJitter(self, **args):
        return self._add(transforms.ColorJitter(**args))

    def RandomRotation(self, **args):
        return self._add(transforms.RandomRotation(**args))

    def RandomAffine(self, **args):
        return self._add(transforms.RandomAffine(**args))

    def Grayscale(self, **args):
        return self._add(transforms.Grayscale(**args))

    def RandomGrayscale(self, **args):
        return self._add(transforms.RandomGrayscale(**args))

    def RandomPerspective(self, **args):
        return self._add(transforms.RandomPerspective(**args))

    def RandomErasing(self, **args):
        return self._add(transforms.RandomErasing(**args))
